package com.yihu.quota.service.medicalInsurance;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.yihu.ehr.hbase.HBaseDao;
import com.yihu.ehr.profile.core.ResourceCore;
import com.yihu.ehr.solr.SolrUtil;
import com.yihu.ehr.util.datetime.DateUtil;
import com.yihu.ehr.util.rest.Envelop;
import com.yihu.quota.controller.QuotaReportController;
import com.yihu.quota.service.quota.BaseStatistsService;
import org.apache.commons.lang.StringUtils;
import org.apache.solr.client.solrj.response.PivotField;
import org.apache.solr.client.solrj.response.RangeFacet;
import org.apache.solr.common.SolrDocument;
import org.apache.solr.common.SolrDocumentList;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author: zhengwei
 * @Date: 2018/9/19 20:35
 * @Description:医保相关
 */
@Service
public class MedicalInsuranceService {
    @Autowired
    private SolrUtil solrUtil;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private ObjectMapper objectMapper;
    @Autowired
    private QuotaReportController quotaReportController;
    @Autowired
    private BaseStatistsService baseStatistsService;
    @Autowired
    private HBaseDao hBaseDao;


    /**
     * 超高费用
     * @param month
     * @param eventType
     * @param hosLevel
     * @param hosHierarchy
     * @param size
     * @param page
     * @return
     * @throws Exception
     */
    public Envelop highCost(String month,String eventType,String hosLevel, String hosHierarchy,int size, int page) throws Exception{
        Envelop result = new Envelop();
        double fee = 0.0;
        String q = "event_type:" + eventType;
        StringBuffer sb = new StringBuffer();
        String start = month+"-01T00:00:00Z";
        String end = DateUtil.getLastDate(month+"-01","yyyy-MM-dd","yyyy-MM-dd")+"T23:59:59Z";
        String[] subfields1 = new String[2];
        String[] subfields2 = new String[2];
        String endDate = DateUtil.getLastDateOfLashMonth(month+"-01","yyyy-MM-dd","yyyy-MM-dd");
        String lastmonth=endDate.substring(0,7);//上月
        String startDate=lastmonth+"-01";
        String filters = "quotaDate >='"+startDate+"' and quotaDate <='"+endDate+"'";
        if("0".equals(eventType)){//门诊
            //获取门诊次均费用
            List<Map<String, Object>> ls= quotaReportController.getQuotaReportTwoDimensionalTable("HC_07_1001",filters,"month","");
            for(Map<String, Object> map:ls){
                if(lastmonth.equals(map.get("month"))){
                    fee=Double.parseDouble(map.get("HC_07_1001")+"")*5;
                    break;
                }
            }
            sb.append("EHR_000045:[").append(fee).append(" TO *]").append(" AND event_date:[")
                    .append(start).append(" TO ").append(end).append("]");
            subfields1[0]="EHR_000045";
            subfields1[1]="profile_id";
            subfields2[0]="EHR_000109";
            subfields2[1]="EHR_000109_VALUE";
        }else{//住院
            //获取住院次均费用
            List<Map<String, Object>> ls= quotaReportController.getQuotaReportTwoDimensionalTable("HC_07_1002",filters,"month","");
            for(Map<String, Object> map:ls){
                if(lastmonth.equals(map.get("month"))){
                    fee=Double.parseDouble(map.get("HC_07_1002")+"")*5;
                    break;
                }
            }
            sb.append("EHR_000175:[").append(fee).append(" TO *]").append(" AND event_date:[")
                    .append(start).append(" TO ").append(end).append("]");
            subfields1[0]="EHR_000175";
            subfields1[1]="profile_id";
            subfields2[0]="EHR_000293";
            subfields2[1]="EHR_000293_VALUE";
        }
        if(StringUtils.isNotEmpty(hosLevel)||StringUtils.isNotEmpty(hosHierarchy)){
            String orgCode=getOrgCode(hosLevel,hosHierarchy);
            sb.append(" AND org_code:(").append(StringUtils.isEmpty(orgCode)? "null":orgCode).append(")");
        }
        String[] fields = {"rowkey","patient_name","patient_age","patient_sex","demographic_id","EHR_001211","event_date"};
        SolrDocumentList sublist = solrUtil.query(ResourceCore.SubTable,q,sb.toString(),null,(page-1)*size,size,subfields1);
        if(sublist!=null && sublist.getNumFound()>0){
            for (SolrDocument doc : sublist){
                String fq="profile_id:"+doc.getFieldValue("profile_id");
                if("0".equals(eventType)){
                    doc.put("fee",doc.getFieldValue("EHR_000045"));//门诊费用
                    SolrDocumentList ls = solrUtil.query(ResourceCore.SubTable,"EHR_000109:*",fq,null,0,1,subfields2);
                    if(ls!=null&&ls.size()>0){
                        String name = ls.get(0).getFieldValue("EHR_000109_VALUE")==null?"":
                                ls.get(0).getFieldValue("EHR_000109_VALUE")+"";
                        doc.put("diagnosis",name+"（"+ls.get(0).getFieldValue("EHR_000109")+"）");//门诊诊断
                    }else{
                        doc.put("diagnosis","");
                    }
                }else{
                    doc.put("fee",doc.getFieldValue("EHR_000175"));
                    SolrDocumentList ls = solrUtil.query(ResourceCore.SubTable,"EHR_000293:*",fq,null,0,1,subfields2);
                    if(ls!=null&&ls.size()>0){
                        String name = ls.get(0).getFieldValue("EHR_000293_VALUE")==null?"":
                                ls.get(0).getFieldValue("EHR_000293_VALUE")+"";
                        doc.put("diagnosis", name+"（"+ls.get(0).getFieldValue("EHR_000293")+"）");
                    }else{
                        doc.put("diagnosis","");
                    }
                }
                String q1="rowkey:"+doc.getFieldValue("profile_id");
                SolrDocumentList list = solrUtil.query(ResourceCore.MasterTable,q1,null,null,0,1,fields);
                doc.putAll(list.get(0));
                doc.put("address", null != list.get(0).get("EHR_001211") ? list.get(0).get("EHR_001211") : "");
                doc.put("event_date", DateUtil.toString(new Date(doc.getFieldValue("event_date")+"")));
            }
        }
        long count = solrUtil.count(ResourceCore.SubTable, q, sb.toString());
        result.setSuccessFlg(true);
        result.setObj((double)Math.round(fee*100)/100);
        result.setDetailModelList(sublist);
        result.setTotalCount((int)count);
        result.setCurrPage(page);
        result.setPageSize(size);
        if (result.getTotalCount() % result.getPageSize() > 0) {
            result.setTotalPage(result.getTotalCount() / result.getPageSize() + 1);
        } else {
            result.setTotalPage(result.getTotalCount() / result.getPageSize());
        }
        return result;
    }

    /**
     * 频繁就医
     * @param time
     * @param size
     * @param page
     * @return
     * @throws Exception
     */
    public Envelop oftenIll(String time, int size, int page) throws Exception {
        Envelop envelop = new Envelop();
        List<Map<String, Object>> list = new ArrayList<>();
        SimpleDateFormat dfs = new SimpleDateFormat("yyyy-MM-dd'T'23:59:59'Z'");
        // idCardList用来存放最近7天就诊次数大于等于3的用户身份证号码
        HashSet<String> idCardList = new HashSet<>();

        // 时间
        Date dateTime = DateUtil.strToDate(time);
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(dateTime);
        // 设置成本月第一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        String startDate = DateUtil.utcToDate(calendar.getTime());
        // 设置成本月最后一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        String endDate = dfs.format(calendar.getTime());

        String demographicId = "demographic_id";
        String[] fields = {"rowkey","patient_name","patient_age","patient_sex","demographic_id","EHR_001211","event_date"};

        // 设置成本月第一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));

        // 遍历一个月
        while (belongCalendar(calendar, startDate, endDate)) {
            Calendar innerCalendar = Calendar.getInstance();
            innerCalendar.setTime(calendar.getTime());
            // 获取该月份下每天前后3天的就诊记录, 共7天
            innerCalendar.add(Calendar.DATE, -3);
            String preDate = DateUtil.utcToDate(innerCalendar.getTime());
            innerCalendar.add(Calendar.DATE, 6);
            String sufDate = dfs.format(innerCalendar.getTime());
            String fq = new StringBuffer().append("event_date:[").append(preDate).append(" TO ").append(sufDate).append("]").toString();

            Map<String, Long> groupMap = solrUtil.groupCountLte(ResourceCore.MasterTable, null, fq, demographicId, 0, -1, 3);
            for (String key : groupMap.keySet()) {
                idCardList.add(key);
            }
            calendar.add(Calendar.DATE, 1);
        }
        // 设置查询的时间段
        calendar.add(Calendar.DATE, 2);
        String end = dfs.format(calendar.getTime());
        calendar.add(Calendar.DATE, -3);
        // 设置成本月第一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        calendar.add(Calendar.DATE, -3);
        String start = DateUtil.utcToDate(calendar.getTime());
        String fqAll = new StringBuffer().append("event_date:[").append(start).append(" TO ").append(end).append("]").toString();

        Map<String, Long> groupMap = solrUtil.groupCountLte(ResourceCore.MasterTable, null, fqAll, demographicId, 0, -1, 3);
        for (String idCard : idCardList) {
            Map<String, Object> mapInfo = new HashMap();
            SolrDocumentList query = solrUtil.query(ResourceCore.MasterTable, "demographic_id:" + idCard, fqAll, null, 0, 1, fields);
            if (null != query && query.size() > 0) {
                String rowKey = null != query.get(0).get("rowkey") ? query.get(0).get("rowkey") + "" : "";
                String sex = null != query.get(0).get("patient_sex") ? query.get(0).get("patient_sex") + "" : "";
                mapInfo.put("rowkey", rowKey);
                mapInfo.put("demographic_id", idCard);
                mapInfo.put("patient_name", null != query.get(0).get("patient_name") ? query.get(0).get("patient_name") : "");
                mapInfo.put("patient_age", null != query.get(0).get("patient_age") ? query.get(0).get("patient_age") : "");
                mapInfo.put("patient_sex", sexExchangeByCode(sex));
                mapInfo.put("result", null != groupMap.get(idCard) ? groupMap.get(idCard) : 3);
                mapInfo.put("address", null != query.get(0).get("EHR_001211") ? query.get(0).get("EHR_001211") : "");
                list.add(mapInfo);
            }
        }
        // 获取总记录数
        int totalCount = list.size();
        // 根据就诊次数排序
        list = baseStatistsService.sortResultList(list, null);

        list = list.subList((page - 1) * size, ((page - 1) * size + size) > list.size() ? list.size() : (page - 1) * size + size);

        envelop.setCurrPage(page);
        envelop.setPageSize(size);
        envelop.setTotalCount(totalCount);
        envelop.setDetailModelList(list);
        // 设置总页码数
        if (envelop.getTotalCount() % envelop.getPageSize() > 0) {
            envelop.setTotalPage(envelop.getTotalCount() / envelop.getPageSize() + 1);
        } else {
            envelop.setTotalPage(envelop.getTotalCount() / envelop.getPageSize());
        }
        envelop.setSuccessFlg(true);
        return envelop;
    }

    private String getOrgCode(String hosLevel, String hosHierarchy){
        String orgCode="";
        StringBuffer sql = new StringBuffer();
        sql.append("select org_code from organizations where org_type='Hospital' ");
        if(StringUtils.isNotEmpty(hosLevel)){
            sql.append(" and level_id ="+hosLevel);
        }
        if(StringUtils.isNotEmpty(hosHierarchy)){
            sql.append(" and hos_hierarchy ="+hosHierarchy);
        }
        List<Map<String,Object>> list = jdbcTemplate.queryForList(sql.toString());
        if(!list.isEmpty()){
            for(Map<String, Object> map:list){
                orgCode+=map.get("ORG_CODE")+" ";
            }
            orgCode=orgCode.substring(0,orgCode.length()-1);
        }
        return orgCode;
    }

    /**
     * 性别字典转换
     * @param code
     * @return
     */
    private String sexExchangeByCode(String code) {
        String sex = "";
        switch (code) {
            case "0" : {
                sex = "未知的性别";
                break;
            }
            case "1" : {
                sex = "男性";
                break;
            }
            case "2" : {
                sex = "女性";
                break;
            }
            case "9" : {
                sex = "未说明的性别";
                break;
            }
            default : {
                break;
            }
        }
        return sex;
    }

    /**
     * 判断是否在指定时间段内
     * @param date
     * @param beginTime
     * @param endTime
     * @return
     */
    public boolean belongCalendar(Calendar date, String beginTime, String endTime) {
        Date start = DateUtil.strToDate(beginTime, DateUtil.utcDateTimePattern);
        Calendar begin = Calendar.getInstance();
        begin.setTime(start);

        Calendar end = Calendar.getInstance();
        Date finish = DateUtil.strToDate(endTime, DateUtil.utcDateTimePattern);
        end.setTime(finish);

        if (date.compareTo(begin) >= 0 && date.compareTo(end) <= 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 超高费用
     * @param month
     * @param inspect
     * @param size
     * @param page
     * @return
     * @throws Exception
     */
    public Envelop repeateInspect(String month, String inspect,int size, int page) throws Exception{
        Envelop envelop = new Envelop();
        SimpleDateFormat dfs = new SimpleDateFormat("yyyy-MM-dd'T'23:59:59'Z'");
        String q = "";
        if(StringUtils.isNotEmpty(inspect)){
            q="EHR_000333:"+inspect;
        }else{
            q="EHR_000333:*";
        }
        List<Map<String,Object>> res = new ArrayList<>();
        // 时间
        Date dateTime = DateUtil.strToDate(month+"-01");
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(dateTime);
        // 设置成本月第一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        String startDate = DateUtil.utcToDate(calendar.getTime());
        // 设置成本月最后一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DAY_OF_MONTH));
        String endDate = dfs.format(calendar.getTime());
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        // 遍历一个月
        while (belongCalendar(calendar, startDate, endDate)) {
            Calendar innerCalendar = Calendar.getInstance();
            innerCalendar.setTime(calendar.getTime());
            // 获取该月份下每天前后3天的就诊记录, 共7天
            innerCalendar.add(Calendar.DATE, -3);
            String preDate = DateUtil.utcToDate(innerCalendar.getTime());
            innerCalendar.add(Calendar.DATE, 6);
            String sufDate = dfs.format(innerCalendar.getTime());
            String fq = new StringBuffer().append("event_date:[").append(preDate).append(" TO ").append(sufDate).append("]").toString();
            List<PivotField> list = solrUtil.groupCountMult(ResourceCore.SubTable,q,fq,"EHR_000017,EHR_000333",0,-1);
            for(PivotField pivotField:list){
                for(PivotField child:pivotField.getPivot()){
                    if(child.getCount()>=3){
                        checkIdCard(res, pivotField.getValue()+"", child.getValue()+"", child.getCount());
                    }
                }
            }
            calendar.add(Calendar.DATE, 1);
        }

        // 设置查询的时间段
        calendar.add(Calendar.DATE, 2);
        String end = dfs.format(calendar.getTime());
        calendar.add(Calendar.DATE, -3);
        // 设置成本月第一天
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
        calendar.add(Calendar.DATE, -3);
        String start = DateUtil.utcToDate(calendar.getTime());
        String fqAll = new StringBuffer().append("event_date:[").append(start).append(" TO ").append(end).append("]").toString();
        String[] fields = {"rowkey","patient_name","patient_age","patient_sex","demographic_id","EHR_001211","event_date"};
        for(Map<String,Object> mapInfo:res){
            SolrDocumentList query = solrUtil.query(ResourceCore.MasterTable, "demographic_id:" + mapInfo.get("demographic_id"), fqAll, null, 0, 1, fields);
            if (null != query && query.size() > 0) {
                String rowKey = null != query.get(0).get("rowkey") ? query.get(0).get("rowkey") + "" : "";
                String sex = null != query.get(0).get("patient_sex") ? query.get(0).get("patient_sex") + "" : "";
                mapInfo.put("rowkey", rowKey);
                mapInfo.put("patient_name", null != query.get(0).get("patient_name") ? query.get(0).get("patient_name") : "");
                mapInfo.put("patient_age", null != query.get(0).get("patient_age") ? query.get(0).get("patient_age") : "");
                mapInfo.put("patient_sex", sexExchangeByCode(sex));
                mapInfo.put("address", null != query.get(0).get("EHR_001211") ? query.get(0).get("EHR_001211") : "");
            }
        }
        // 获取总记录数
        int totalCount = res.size();

        res = res.subList((page - 1) * size, ((page - 1) * size + size) > res.size() ? res.size() : (page - 1) * size + size);
        envelop.setCurrPage(page);
        envelop.setPageSize(size);
        envelop.setTotalCount(totalCount);
        envelop.setDetailModelList(res);
        // 设置总页码数
        if (envelop.getTotalCount() % envelop.getPageSize() > 0) {
            envelop.setTotalPage(envelop.getTotalCount() / envelop.getPageSize() + 1);
        } else {
            envelop.setTotalPage(envelop.getTotalCount() / envelop.getPageSize());
        }
        envelop.setSuccessFlg(true);
        return envelop;
    }

    public void checkIdCard(List<Map<String,Object>> list ,String idCard, String inspectCode, int count){
        boolean flag = false;
        for(Map<String,Object> map: list){
            if(idCard.equals(map.get("demographic_id"))&&inspectCode.equals(map.get("inspectCode"))){
                map.put("result",Integer.parseInt(map.get("result")+"")+count);
                flag=true;
                break;
            }
        }
        if(!flag){
            Map<String,Object> resMap = new HashMap<>();
            resMap.put("demographic_id",idCard);
            resMap.put("inspectCode",inspectCode);
            resMap.put("result",count);
            list.add(resMap);
        }
    }
    /**
     * 用药分析
     * @param start
     * @param end
     * @param eventType
     * @param drugName
     * @param size
     * @param page
     * @return
     * @throws Exception
     */
    public Envelop drugAnalysis(String start,String end,String eventType,String drugName,int size, int page) throws Exception{
        Envelop result = new Envelop();
        String q = "";
        StringBuffer sb = new StringBuffer();
        if(StringUtils.isNotEmpty(drugName)){
            q = "EHR_000100:*"+drugName+"* OR EHR_000131:*"+drugName+"*";
        }else{
            q = "EHR_000100:* OR EHR_000131:*";
        }
        start = start+"T00:00:00Z";
        end = end+"T00:00:00Z";
        sb.append("event_date:[").append(start).append(" TO ").append(end).append("]");
        if(StringUtils.isNotEmpty(eventType)){
            sb.append(" AND event_type:"+eventType);
        }
        String[] subfields = new String[3];
        subfields[0]="profile_id";
        subfields[1]="event_type";
        subfields[2]="rowkey";
        String[] fi1elds = {"rowkey","patient_name","patient_age","patient_sex","demographic_id","EHR_001211","event_date"};
        SolrDocumentList sublist = solrUtil.query(ResourceCore.SubTable,q,sb.toString(),null,(page-1)*size,size,subfields);
        if(sublist!=null && sublist.getNumFound()>0){
            for (SolrDocument doc : sublist){
                String subRowkey = String.valueOf(doc.getFieldValue("rowkey"));
                Map<String, Object> map = hBaseDao.getResultMap(ResourceCore.SubTable, subRowkey);
                if(subRowkey.indexOf("HDSD00_84")!=-1){
                    doc.put("HDSD00_04_023",map.get("EHR_000100"));//西药名称
                }else{
                    doc.put("HDSD00_04_023",map.get("EHR_000131"));//中药名称
                }
                doc.put("HDSD00_04_021",map.get("EHR_000129"));//规格
                doc.put("HDSD00_04_027",map.get("EHR_000136")+" 每次"+ map.get("EHR_000101")+
                        map.get("EHR_000133")+" "+map.get("EHR_000134"));//用法
                doc.put("HDSD00_04_028",map.get("EHR_000104"));//总量
                String[] subfields1 = new String[1];
                String[] subfields2 = new String[2];
                String fq="profile_id:"+doc.getFieldValue("profile_id");
                if("0".equals(map.get("event_type"))){
                    subfields1[0]="EHR_000045";
                    subfields2[0]="EHR_000109";
                    subfields2[1]="EHR_000109_VALUE";
                    SolrDocumentList ls1 = solrUtil.query(ResourceCore.SubTable,"EHR_000109:*",fq,null,0,1,subfields2);
                    SolrDocumentList ls2 = solrUtil.query(ResourceCore.SubTable,"EHR_000045:*",fq,null,0,1,subfields1);
                    if(ls1!=null&&ls1.size()>0){
                        String name = ls1.get(0).getFieldValue("EHR_000109_VALUE")==null?"": ls1.get(0).getFieldValue("EHR_000109_VALUE")+"";
                        doc.put("diagnosis",name+"（"+ls1.get(0).getFieldValue("EHR_000109")+"）");
                    }else{
                        doc.put("diagnosis","");
                    }
                    if(ls2!=null&&ls2.size()>0){
                        doc.put("fee",ls2.get(0).getFieldValue("EHR_000045"));//门诊费用
                    }else{
                        doc.put("fee","");
                    }

                }else if("1".equals(map.get("event_type"))){
                    subfields1[0]="EHR_000175";
                    subfields2[0]="EHR_000293";
                    subfields2[1]="EHR_000293_VALUE";
                    SolrDocumentList ls1 = solrUtil.query(ResourceCore.SubTable,"EHR_000293:*",fq,null,0,1,subfields2);
                    SolrDocumentList ls2 = solrUtil.query(ResourceCore.SubTable,"EHR_000175:*",fq,null,0,1,subfields1);
                    if(ls1!=null&&ls1.size()>0){
                        String name = ls1.get(0).getFieldValue("EHR_000293_VALUE")==null?"":ls1.get(0).getFieldValue("EHR_000293_VALUE")+"";
                        doc.put("diagnosis",name+"（"+ls1.get(0).getFieldValue("EHR_000293")+"）");
                    }else{
                        doc.put("diagnosis","");
                    }
                    if(ls2!=null&&ls2.size()>0){
                        doc.put("fee",ls2.get(0).getFieldValue("EHR_000175"));//门诊费用
                    }else{
                        doc.put("fee","");
                    }
                }
                String q1="rowkey:"+doc.getFieldValue("profile_id");
                SolrDocumentList list = solrUtil.query(ResourceCore.MasterTable,q1,null,null,0,1,fi1elds);
                doc.putAll(list.get(0));
                doc.put("address", null != list.get(0).get("EHR_001211") ? list.get(0).get("EHR_001211") : "");
                doc.put("event_date", DateUtil.toString(new Date(doc.getFieldValue("event_date")+"")));
            }
        }
        long count = solrUtil.count(ResourceCore.SubTable, q, sb.toString());
        result.setSuccessFlg(true);
        result.setDetailModelList(sublist);
        result.setTotalCount((int)count);
        result.setCurrPage(page);
        result.setPageSize(size);
        if (result.getTotalCount() % result.getPageSize() > 0) {
            result.setTotalPage(result.getTotalCount() / result.getPageSize() + 1);
        } else {
            result.setTotalPage(result.getTotalCount() / result.getPageSize());
        }
        return result;
    }
}
